import uuid

from datetime import date, datetime, timedelta
from typing import List

from fastapi import APIRouter, Cookie, Depends, HTTPException, Security, status
from sqlalchemy import func
from sqlalchemy.orm import Session

from ...core.config import settings
from ...core.security import get_current_active_user as get_user
from ...db.session import SessionLocal
from ...models import Inventory, Kot, Product, SaleCategory, Voucher, VoucherType
from ...printing.discount_report import print_discount_report
from ...schemas.auth import UserToken
from ...schemas.discount_report import DiscountReport, DiscountReportItem


router = APIRouter()


# Dependency
def get_db() -> Session:
    try:
        db = SessionLocal()
        yield db
    finally:
        db.close()


@router.get("", response_model=DiscountReport)
def discount_report(
    s: str = None,
    f: str = None,
    db: Session = Depends(get_db),
    user: UserToken = Security(get_user, scopes=["discount-report"]),
) -> DiscountReport:
    start_date = date.today() if s is None else datetime.strptime(s, "%d-%b-%Y").date()
    finish_date = date.today() if f is None else datetime.strptime(f, "%d-%b-%Y").date()
    if (date.today() - start_date).days > 5 and "accounts-audit" not in user.permissions:
        raise HTTPException(
            status_code=status.HTTP_403_FORBIDDEN,
            detail="Accounts Audit",
        )
    return DiscountReport(
        startDate=start_date,
        finishDate=finish_date,
        amounts=get_discount_report(start_date, finish_date, db),
    )


def get_discount_report(s: date, f: date, db: Session) -> List[DiscountReportItem]:
    start_date = s + timedelta(minutes=settings.NEW_DAY_OFFSET_MINUTES)
    finish_date = f + timedelta(days=1, minutes=settings.NEW_DAY_OFFSET_MINUTES)

    amount = func.sum(Inventory.quantity * Inventory.effective_price * Inventory.discount).label("Amount")
    list_ = (
        db.query(SaleCategory.name, amount)
        .join(Voucher.kots)
        .join(Kot.inventories)
        .join(Inventory.product)
        .join(Product.sale_category)
        .filter(
            Inventory.discount != 0,
            Voucher.date >= start_date,
            Voucher.date <= finish_date,
            Voucher.voucher_type.in_([VoucherType.REGULAR_BILL.value, VoucherType.KOT.value]),
        )
        .group_by(SaleCategory.name)
        .order_by(SaleCategory.name)
        .all()
    )

    return [DiscountReportItem(name=pg, amount=amt) for pg, amt in list_]


@router.get("/print", response_model=bool)
def print_report(
    s: str = None,
    f: str = None,
    device_id: uuid.UUID = Cookie(None),
    db: Session = Depends(get_db),
    user: UserToken = Security(get_user, scopes=["discount-report"]),
) -> bool:
    start_date = date.today() if s is None else datetime.strptime(s, "%d-%b-%Y").date()
    finish_date = date.today() if f is None else datetime.strptime(f, "%d-%b-%Y").date()
    if (date.today() - start_date).days > 5 and "accounts-audit" not in user.permissions:
        raise HTTPException(
            status_code=status.HTTP_403_FORBIDDEN,
            detail="Accounts Audit",
        )
    report = DiscountReport(
        startDate=start_date,
        finishDate=finish_date,
        amounts=get_discount_report(start_date, finish_date, db),
    )
    print_discount_report(report, device_id, db)
    return True
