import uuid

from typing import Optional

import barker.schemas.master as schemas

from fastapi import APIRouter, Depends, HTTPException, Security, status
from sqlalchemy.exc import SQLAlchemyError
from sqlalchemy.orm import Session

from ..core.security import get_current_active_user as get_user
from ..db.session import SessionLocal
from ..models.master import SaleCategory
from ..schemas.auth import UserToken


router = APIRouter()


# Dependency
def get_db():
    try:
        db = SessionLocal()
        yield db
    finally:
        db.close()


@router.post("", response_model=schemas.SaleCategory)
def save(
    data: schemas.SaleCategoryIn,
    db: Session = Depends(get_db),
    user: UserToken = Security(get_user, scopes=["products"]),
):
    try:
        item = SaleCategory(name=data.name, tax_id=data.tax.id_)
        db.add(item)
        db.commit()
        return sale_category_info(item)
    except SQLAlchemyError as e:
        db.rollback()
        raise HTTPException(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            detail=str(e),
        )
    except Exception:
        db.rollback()
        raise


@router.put("/{id_}", response_model=schemas.SaleCategory)
def update(
    id_: uuid.UUID,
    data: schemas.SaleCategoryIn,
    db: Session = Depends(get_db),
    user: UserToken = Security(get_user, scopes=["products"]),
):
    try:
        item: SaleCategory = db.query(SaleCategory).filter(SaleCategory.id == id_).first()
        item.name = data.name
        item.tax_id = data.tax.id_
        db.commit()
        return sale_category_info(item)
    except SQLAlchemyError as e:
        db.rollback()
        raise HTTPException(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            detail=str(e),
        )
    except Exception:
        db.rollback()
        raise


@router.delete("/{id_}")
def delete(
    id_: uuid.UUID,
    db: Session = Depends(get_db),
    user: UserToken = Security(get_user, scopes=["products"]),
):
    try:
        item: SaleCategory = db.query(SaleCategory).filter(SaleCategory.id == id_).first()
        if item is None:
            raise HTTPException(
                status_code=status.HTTP_404_NOT_FOUND,
                detail="Sale Category not found",
            )
        else:
            raise HTTPException(
                status_code=status.HTTP_501_NOT_IMPLEMENTED,
                detail="Sale Category deletion not implemented",
            )
    except Exception:
        db.rollback()
        raise


@router.get("")
def show_blank(
    db: Session = Depends(get_db),
    user: UserToken = Security(get_user, scopes=["products"]),
):
    return sale_category_info(None)


@router.get("/list")
def show_list(db: Session = Depends(get_db), user: UserToken = Depends(get_user)):
    return [sale_category_info(item) for item in db.query(SaleCategory).order_by(SaleCategory.name).all()]


@router.get("/for-discount")
def for_discount(db: Session = Depends(get_db), user: UserToken = Depends(get_user)):
    return [
        {"id": item.id, "name": item.name, "discount": 0}
        for item in db.query(SaleCategory).order_by(SaleCategory.name).all()
    ]


@router.get("/{id_}")
def show_id(
    id_: uuid.UUID,
    db: Session = Depends(get_db),
    user: UserToken = Security(get_user, scopes=["products"]),
):
    item: SaleCategory = db.query(SaleCategory).filter(SaleCategory.id == id_).first()
    return sale_category_info(item)


def sale_category_info(item: Optional[SaleCategory]):
    if item is None:
        return {"name": "", "tax": {}}
    return {
        "id": item.id,
        "name": item.name,
        "tax": {"id": item.tax_id, "name": item.tax.name, "rate": item.tax.rate},
    }
