import { Component, OnInit } from '@angular/core';
import { FormBuilder, FormGroup } from '@angular/forms';
import { ActivatedRoute, Router } from '@angular/router';
import * as moment from 'moment';

import { ToCsvService } from '../shared/to-csv.service';

import { BeerConsumptionReport } from './beer-consumption-report';
import { BeerConsumptionReportDataSource } from './beer-consumption-report-datasource';

@Component({
  selector: 'app-beer-consumption-report',
  templateUrl: './beer-consumption-report.component.html',
  styleUrls: ['./beer-consumption-report.component.css'],
})
export class BeerConsumptionReportComponent implements OnInit {
  dataSource: BeerConsumptionReportDataSource;
  form: FormGroup;
  info: BeerConsumptionReport;

  /** Columns displayed in the table. Columns IDs can be added, removed, or reordered. */
  displayedColumns: string[] = ['date'];

  constructor(
    private route: ActivatedRoute,
    private router: Router,
    private fb: FormBuilder,
    private toCsv: ToCsvService,
  ) {
    this.createForm();
  }

  ngOnInit() {
    this.route.data.subscribe((data: { info: BeerConsumptionReport }) => {
      this.info = data.info;
      this.displayedColumns = ['date'].concat(this.info.headers);
      this.form.setValue({
        startDate: moment(this.info.startDate, 'DD-MMM-YYYY').toDate(),
        finishDate: moment(this.info.finishDate, 'DD-MMM-YYYY').toDate(),
      });
      this.dataSource = new BeerConsumptionReportDataSource(this.info.data);
    });
  }

  show() {
    const info = this.getInfo();
    this.router.navigate(['beer-consumption-report'], {
      queryParams: {
        startDate: info.startDate,
        finishDate: info.finishDate,
      },
    });
  }

  createForm() {
    this.form = this.fb.group({
      startDate: '',
      finishDate: '',
    });
  }

  getInfo(): BeerConsumptionReport {
    const formModel = this.form.value;

    return {
      startDate: moment(formModel.startDate).format('DD-MMM-YYYY'),
      finishDate: moment(formModel.finishDate).format('DD-MMM-YYYY'),
    };
  }

  exportCsv() {
    const headers = this.info.headers.reduce(
      (a, c) => {
        a[c] = c;
        return a;
      },
      {
        Date: 'date',
      },
    );
    const csvData = new Blob([this.toCsv.toCsv(headers, this.info.data)], {
      type: 'text/csv;charset=utf-8;',
    });
    const link = document.createElement('a');
    link.href = window.URL.createObjectURL(csvData);
    link.setAttribute('download', 'beer-consumption-report.csv');
    document.body.appendChild(link);
    link.click();
    document.body.removeChild(link);
  }
}
