import { Component, OnInit } from '@angular/core';
import { FormBuilder, FormGroup } from '@angular/forms';
import { MatDialog } from '@angular/material/dialog';
import { ActivatedRoute, Router } from '@angular/router';
import * as moment from 'moment';
import { BehaviorSubject } from 'rxjs';
import { map } from 'rxjs/operators';

import { ToasterService } from '../../core/toaster.service';
import { ConfirmDialogComponent } from '../../shared/confirm-dialog/confirm-dialog.component';
import { GuestBook } from '../guest-book';
import { GuestBookList } from '../guest-book-list';
import { GuestBookService } from '../guest-book.service';

import { GuestBookListDataSource } from './guest-book-list-datasource';

@Component({
  selector: 'app-guest-book-list',
  templateUrl: './guest-book-list.component.html',
  styleUrls: ['./guest-book-list.component.css'],
})
export class GuestBookListComponent implements OnInit {
  dataSource: GuestBookListDataSource;
  form: FormGroup;
  data: BehaviorSubject<GuestBook[]>;
  /** Columns displayed in the table. Columns IDs can be added, removed, or reordered. */
  displayedColumns = ['sno', 'name', 'phone', 'pax', 'date', 'action'];

  constructor(
    private route: ActivatedRoute,
    private router: Router,
    private fb: FormBuilder,
    private dialog: MatDialog,
    private toaster: ToasterService,
    private ser: GuestBookService,
  ) {
    this.createForm();
    this.data = new BehaviorSubject([]);
    this.listenToDateChange();
  }

  createForm() {
    this.form = this.fb.group({
      date: '',
    });
  }

  listenToDateChange(): void {
    this.form
      .get('date')
      .valueChanges.pipe(map((x) => moment(x).format('DD-MMM-YYYY')))
      .subscribe((x) => {
        return this.ser.list(x).subscribe((list: GuestBookList) => {
          this.data.next(list.list);
        });
      });
  }

  ngOnInit() {
    this.route.data.subscribe((data: { list: GuestBookList }) => {
      this.data.next(data.list.list);
      this.form.setValue({ date: moment(data.list.date, 'DD-MMM-YYYY').toDate() });
    });
    this.dataSource = new GuestBookListDataSource(this.data);
  }

  delete(id: string) {
    this.ser.delete(id).subscribe(
      () => {
        this.toaster.show('Success', '');
        this.router.navigateByUrl('/guest-book');
      },
      (error) => {
        this.toaster.show('Danger', error);
      },
    );
  }

  confirmDelete(id: string): void {
    const dialogRef = this.dialog.open(ConfirmDialogComponent, {
      width: '250px',
      data: { title: 'Delete Guest Book Entry?', content: 'Are you sure? This cannot be undone.' },
    });

    dialogRef.afterClosed().subscribe((result: boolean) => {
      if (result) {
        this.delete(id);
      }
    });
  }
}
